@@ -5,25 +5,51 @@ use warnings;
 use base 'Module::Build';
 
 use lib "inc";
-use My::Utility qw(find_SDL_dir find_file sed_inplace);
+use My::Utility qw(find_SDL_dir find_file sed_inplace get_dlext);
 use File::Spec::Functions qw(catdir catfile splitpath catpath rel2abs abs2rel);
 use File::Path qw(make_path remove_tree);
 use File::Copy qw(cp);
 use File::Fetch;
 use File::Find;
+use File::ShareDir;
 use Archive::Extract;
 use Digest::SHA qw(sha1_hex);
+use Text::Patch;
 use Config;
 
 sub ACTION_build {
   my $self = shift;
   # as we want to wipe 'sharedir' during 'Build clean' we has
   # to recreate 'sharedir' at this point if it does not exist
+
+  my $bp = $self->notes('build_params');
+  die "###ERROR### Cannot continue build_params not defined" unless defined($bp);
+
+  printf("Build option used:\n\t%s\n", $bp->{title} || 'n.a.');
+
   mkdir 'sharedir' unless(-d 'sharedir');
   $self->add_to_cleanup('sharedir');
   $self->SUPER::ACTION_build;
 }
 
+
+sub ACTION_install
+{
+ my $self = shift;
+ my $sharedir = '';
+ 
+ $sharedir = eval {File::ShareDir::dist_dir('Alien-SDL')} || '';
+
+ if ( -d $sharedir )
+ {
+   print "Removing the old $sharedir \n";
+   remove_tree($sharedir);
+   make_path($sharedir);
+ }
+
+ $self->SUPER::ACTION_install;
+}
+
 sub ACTION_code {
   my $self = shift;
 
@@ -52,14 +78,14 @@ sub ACTION_code {
     $self->config_data('build_os', $^O);
     $self->config_data('script', '');            # just to be sure
     $self->config_data('config', {});            # just to be sure
-    $self->config_data('additional_cflafs', ''); # just to be sure
+    $self->config_data('additional_cflags', ''); # just to be sure
     $self->config_data('additional_libs', '');   # just to be sure
 
     if($bp->{buildtype} eq 'use_config_script') {
       $self->config_data('script', $bp->{script});
       # include path trick - adding couple of addititonal locations
-      $self->config_data('additional_cflafs', '-I'.$bp->{prefix}.'/include/smpeg '.
-                                              '-I'.$bp->{prefix}.'/include ' .
+      $self->config_data('additional_cflags', '-I' . $self->get_path($bp->{prefix} . '/include/smpeg') . ' '.
+                                              '-I' . $self->get_path($bp->{prefix} . '/include') . ' ' .
                                               $self->get_additional_cflags);
       $self->config_data('additional_libs', $self->get_additional_libs);
     }
@@ -88,18 +114,37 @@ sub ACTION_code {
 
 sub fetch_file {
   my ($self, $url, $sha1sum, $download) = @_;
-  die "###ERROR### _fetch_file undefined url\n" unless $url;
+  die "###ERROR### _fetch_file undefined url\n"     unless @{$url}[0];
   die "###ERROR### _fetch_file undefined sha1sum\n" unless $sha1sum;
-  my $ff = File::Fetch->new(uri => $url);
+
+  # setting http_proxy environment var if we are within CPAN and this information is available
+  unless($ENV{http_proxy} && $ENV{PERL5_CPAN_IS_RUNNING}) {
+    if(eval('require CPAN::Config; 1') && $CPAN::Config->{http_proxy}) {
+      $ENV{http_proxy} = $CPAN::Config->{http_proxy};
+      if($CPAN::Config->{proxy_user} && $CPAN::Config->{proxy_pass} && $CPAN::Config->{http_proxy} !~ m|//.+:.+@|) {
+        $ENV{http_proxy} =~ s|://|://\Q$CPAN::Config->{proxy_user}\E:\Q$CPAN::Config->{proxy_pass}\E@|;
+      }
+    }
+  }
+
+  my $ff = File::Fetch->new(uri => @{$url}[0]);
   my $fn = catfile($download, $ff->file);
   if (-e $fn) {
     print "Checking checksum for already existing '$fn'...\n";
     return 1 if $self->check_sha1sum($fn, $sha1sum);
     unlink $fn; #exists but wrong checksum
   }
-  print "Fetching '$url'...\n";
-  my $fullpath = $ff->fetch(to => $download);
-  die "###ERROR### Unable to fetch '$url'" unless $fullpath;
+
+  my $fullpath;
+  foreach my $current_url (@{$url})
+  {
+    die "###ERROR### _fetch_file undefined url\n" unless $current_url;
+    print "Fetching '$current_url'...\n";
+    $ff = File::Fetch->new(uri => $current_url);
+    $fullpath = $ff->fetch(to => $download);
+    last if $fullpath;
+  }
+  die "###ERROR### Unable to fetch '$ff->file'" unless $fullpath;
   if (-e $fn) {
     print "Checking checksum for '$fn'...\n";
     return 1 if $self->check_sha1sum($fn, $sha1sum);
@@ -125,7 +170,7 @@ sub extract_binaries {
 
   # do extract binaries
   my $bp = $self->notes('build_params');
-  my $archive = catfile($download, File::Fetch->new(uri => $bp->{url})->file);
+  my $archive = catfile($download, File::Fetch->new(uri => @{$bp->{url}}[0])->file);
   print "Extracting $archive...\n";
   my $ae = Archive::Extract->new( archive => $archive );
   die "###ERROR###: Cannot extract $archive ", $ae->error unless $ae->extract(to => $build_out);
@@ -141,19 +186,31 @@ sub extract_sources {
   foreach my $pack (@{$bp->{members}}) {
     my $srcdir = catfile($build_src, $pack->{dirname});
     my $unpack = 'y';
-    $unpack = $self->prompt("Dir '$srcdir' exists, wanna replace with clean sources?", "n") if (-d $srcdir);
+    $unpack = $self->prompt("Dir '$srcdir' exists, wanna replace with clean sources?", "y") if (-d $srcdir);
     if (lc($unpack) eq 'y') {
-      my $archive = catfile($download, File::Fetch->new(uri => $pack->{url})->file);
+      my $archive = catfile($download, File::Fetch->new(uri => @{$pack->{url}}[0])->file);
       print "Extracting $pack->{pack}...\n";
       my $ae = Archive::Extract->new( archive => $archive );
       die "###ERROR###: cannot extract $pack ", $ae->error unless $ae->extract(to => $build_src);
       foreach my $i (@{$pack->{patches}}) {
-        chdir catfile($build_src, $pack->{dirname});
+        chdir $srcdir;
+        my $patch_file = File::Spec->abs2rel( catfile($patches, $i), $srcdir );
         print "Applying patch '$i'\n";
-        my $cmd = $self->patch_command($srcdir, catfile($patches, $i));
-        if ($cmd) {
-          print "(cmd: $cmd)\n";
-          $self->do_system($cmd) or die "###ERROR### [$?] during patch ... ";
+        foreach my $k ($self->patch_get_affected_files($patch_file)) {
+          # doing the same like -p1 for 'patch'
+          $k =~ s/^[^\/]*\/(.*)$/$1/;
+          open(SRC, $k) or die "###ERROR### Cannot open file: '$k'\n";
+          my @src = <SRC>;
+          close(SRC);
+          open(DIFF, $patch_file) or die "###ERROR### Cannot open file: '$patch_file'\n";
+          my @diff = <DIFF>;
+          close(DIFF);
+          foreach(@src)  { $_=~ s/[\r\n]+$//; }
+          foreach(@diff) { $_=~ s/[\r\n]+$//; }
+          my $out = Text::Patch::patch( join("\n", @src) . "\n", join("\n", @diff) . "\n", { STYLE => "Unified" } );
+          open(OUT, ">$k") or die "###ERROR### Cannot open file for writing: '$k'\n";
+          print(OUT $out);
+          close(OUT);
         }
         chdir $self->base_dir();
       }
@@ -175,8 +232,8 @@ sub set_config_data {
     # defaults
     version     => $version,
     prefix      => '@PrEfIx@',
-    libs        => '-L@PrEfIx@/lib -lSDLmain -lSDL',
-    cflags      => '-I@PrEfIx@/include/SDL -D_GNU_SOURCE=1 -Dmain=SDL_main',
+    libs        => '-L' . $self->get_path('@PrEfIx@/lib') . ' -lSDLmain -lSDL',
+    cflags      => '-I' . $self->get_path('@PrEfIx@/include/SDL') . ' -D_GNU_SOURCE=1 -Dmain=SDL_main',
     shared_libs => [ ],
   };
 
@@ -193,8 +250,21 @@ sub set_config_data {
     }
   }
 
+  my $dlext = get_dlext();
+  # find ld_shared_libs and create symlinks if necessary
+  my $symlink_exists = eval { symlink("",""); 1 };
+  if($symlink_exists)
+  {
+    my @shlibs_ = find_file($build_out, qr/\.$dlext[\d\.]+$/);
+    foreach my $full (@shlibs_){
+      $full =~ qr/(.*\.$dlext)[\d\.]+$/;
+      my ($v, $d, $f) = splitpath($full);
+      symlink("./$f", $1) unless -e $1;
+    }
+  }
+
   # find and set ld_shared_libs
-  my @shlibs = find_file($build_out, qr/\.\Q$Config{dlext}\E$/);
+  my @shlibs = find_file($build_out, qr/\.$dlext$/);
   my $p = rel2abs($prefix);
   $_ =~ s/^\Q$prefix\E/\@PrEfIx\@/ foreach (@shlibs);
   $cfg->{ld_shared_libs} = [ @shlibs ];
@@ -207,29 +277,29 @@ sub set_config_data {
     $tmp{ catpath($v, $d, '') } = 1;
     # available shared libs detection
     if ($f =~ /smpeg/) {
-      $shlib_map{smpeg} = $full;
+      $shlib_map{smpeg} = $full unless $shlib_map{smpeg};
     }
     elsif ($f =~ /^(lib)?(png12)/) {
-      $shlib_map{png12} = $full;
+      $shlib_map{png12} = $full unless $shlib_map{png12};
     }
     elsif ($f =~ /^(lib)?(tiff|jpeg|png)[^a-zA-Z]/) {
-      $shlib_map{$2} = $full;
+      $shlib_map{$2} = $full unless $shlib_map{$2};
     }
     elsif ($f =~ /^(lib)?(SDL_[a-zA-Z]{2,8})[^a-zA-Z0-9]/) {
       # sort of dark magic how to detect SDL_<something> related shlib
-      $shlib_map{$2} = $full;
+      $shlib_map{$2} = $full unless $shlib_map{$2};
     }
     elsif ($f =~ /^(lib)?(SDL)/) {
       # '*SDL*' that did not pass previous test is probably core 'SDL'
-      $shlib_map{SDL} = $full;
+      $shlib_map{SDL} = $full unless $shlib_map{SDL};
     }
   };
   $cfg->{ld_paths} = [ keys %tmp ];
   $cfg->{ld_shlib_map} = \%shlib_map;
 
   # write config
-  $self->config_data('additional_cflafs', '-I@PrEfIx@/include ' .
-                                          '-I@PrEfIx@/include/smpeg ' .
+  $self->config_data('additional_cflags', '-I' . $self->get_path('@PrEfIx@/include') . ' ' .
+                                          '-I' . $self->get_path('@PrEfIx@/include/smpeg') . ' ' .
                                           $self->get_additional_cflags);
   $self->config_data('additional_libs', $self->get_additional_libs);
   $self->config_data('config', $cfg);
@@ -259,6 +329,12 @@ sub get_additional_libs {
   return '';
 }
 
+sub get_path {
+  # this needs to be overriden in My::Builder::<platform>
+  my ( $self, $path ) = @_;
+  return $path;
+}
+
 sub clean_dir {
   my( $self, $dir ) = @_;
   if (-d $dir) {
@@ -296,18 +372,12 @@ sub check_sha1sum {
   return ($sha1->hexdigest eq $sha1sum) ? 1 : 0
 }
 
-sub patch_command {
-  my( $self, $base_dir, $patch_file ) = @_;
-  my $devnull = File::Spec->devnull();
-  my $test = `patch --help 2> $devnull`;
-  if ($test) {
-    $patch_file = File::Spec->abs2rel( $patch_file, $base_dir );
-    # the patches are expected with UNIX newlines
-    # the following command works on both UNIX+Windows
-    return "$^X -pe '' -- $patch_file | patch -N -p1 -u";
-  }
-  warn "###WARN### patch not available";
-  return '';
+sub patch_get_affected_files {
+  my( $self, $patch_file ) = @_;
+  open(DAT, $patch_file) or die "###ERROR### Cannot open file: '$patch_file'\n";
+  my @affected_files = map{$_ =~ /^---\s*([\S]+)/} <DAT>;
+  close(DAT);
+  return @affected_files;
 }
 
 1;
